/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/healthcheck/HealthCheckArtifactDetailsContrib', [ 'Nexus/healthcheck/HealthCheckProUtil' ], function() {

  HealthCheckArtifactDetailsContrib = function() {
      var me = this;

      me.getLoadingHtml = function() {
          return '<div style="background:#ececec; width: 100%; height: 100%; padding: 20px;font-size: 14px;font-family: Arial, Helvetica, sans-serif">Loading...</div>';
      };

      me.getFileUrl = function() {
          return Sonatype.config.servicePath + '/healthCheckFileDetail/' + Sonatype.utils.base64.encode(me.data.resourceURI) + '/index.html';
      };

      me.getGavUrl = function() {
          return Sonatype.config.servicePath + '/healthCheckArtifactDetail/' + me.data.groupId + '/' + me.data.artifactId + '/' + me.data.version + '/index.html';
      };

      me.buildLoadingPanel = function() {
          return new Ext.Panel({
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              html : me.getLoadingHtml()
          });
      };

      me.buildDetailPanel = function(config) {
          var config = config || {};
          var defaultConfig = {
              width : '100%',
              height : '100%',
              border : true,
              collapsible : false,
              collapsed : false,
              layout : 'card',
              items : [ me.buildLoadingPanel() ]
          };

          Ext.applyIf(config, defaultConfig);
          return new Ext.Panel(config);
      };

      me.rebuildIframe = function(panel, url) {
          //item 0 is the loading panel, don't ever remove that one
          panel.getLayout().setActiveItem(0);

          var old = panel.items.get(1);
          if (old) {
              panel.remove(old);
              old.destroy();
          }

          panel.add({
              xtype : 'iframepanel',
              border : false,
              disableMessaging : false,
              defaultSrc : url,
              listeners : {
                  domready : function(frame) {
                      panel.getLayout().setActiveItem(1);
                  }
              }
          });

          panel.doLayout();
      }

      me.viewDetails = function() {
          if (me.artifactPanel && me.data && me.data.groupId && me.data.artifactId && me.data.version) {
              me.rebuildIframe(me.artifactPanel, me.getGavUrl());
          } else if (me.filePanel && me.data && me.data.resourceURI) {
              me.rebuildIframe(me.filePanel, me.getFileUrl());
          }
      };

      me.allowView = function() {
          if (!Sonatype.HealthCheckProUtil.hasAlertReadPerm() || !Sonatype.HealthCheckProUtil.hasPaidProLicense()) {
              return false;
          }

          return true;
      };

      me.showListener = function() {
          me.shown = true;

          if (!me.dataDisplayed && me.data) {
              me.dataDisplayed = true;
              me.viewDetails();
          }
      };

      me.hideListener = function() {
          me.shown = false;
      };

      Sonatype.Events.addListener('fileContainerInit', function(items) {
          if (me.allowView()) {
              me.filePanel = me.buildDetailPanel({
                  name : 'healthCheckFileDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.filePanel);
          }
      });

      Sonatype.Events.addListener('fileContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });

      Sonatype.Events.addListener('artifactContainerInit', function(items) {
          if (me.allowView()) {
              me.artifactPanel = me.buildDetailPanel({
                  name : 'healthCheckArtifactDetailPanel',
                  tabTitle : 'Component Info',
                  preferredIndex : 80,
                  listeners : {
                      show : me.showListener,
                      hide : me.hideListener
                  }
              });
              items.push(me.artifactPanel);
          }
      });

      Sonatype.Events.addListener('artifactContainerUpdate', function(artifactContainer, data) {
          if (me.allowView()) {
              me.data = data;
              me.dataDisplayed = false;

              if (me.shown) {
                  me.dataDisplayed = true;
                  me.viewDetails();
              }
          }
      });
  };

  Ext.onReady(function() {
      Sonatype.HealthCheckArtifactDetailsContrib = new HealthCheckArtifactDetailsContrib();
  });
});