/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Sonatype, Nexus, Ext*/

define('Nexus/usertoken', [
  'Nexus/config', // Sonatype.config
  'Nexus/siesta',
  'Nexus/messages'
], function () {

  // FIXME: Replace with singleton class

  Ext.namespace('Nexus.usertoken');

  /**
   * Base identifier for all plugins bits.
   *
   * @type {String}
   * @static
   */
  Nexus.usertoken.ID = 'nx-usertoken';

  /**
   * Title for various bits.
   *
   * @type {String}
   * @static
   */
  Nexus.usertoken.TITLE = 'User Token';

  /**
   * Show a user-token transient message.
   *
   * @param message
   * @static
   */
  Nexus.usertoken.showMessage = function (message) {
    Nexus.messages.show(Nexus.usertoken.TITLE, message);
  };

  /**
   * Returns the path to a user-token REST resource.
   *
   * @param path
   * @return {String}
   * @static
   */
  Nexus.usertoken.resourcePath = function (path) {
    if (!path.startsWith('/')) {
      path = '/' + path;
    }
    return Nexus.siesta.basePath + '/usertoken' + path;
  };

  /**
   * Flag to indicate if user-token features are enabled in the UI or not.
   *
   * @type {Boolean}
   * @static
   */
  Nexus.usertoken.enabled = false;

  /**
   * Modify user-token enabled flag.
   *
   * @param enabled
   * @static
   */
  Nexus.usertoken.setEnabled = function (enabled) {
    Nexus.log('Nexus.usertoken.enabled: ' + enabled);
    Nexus.usertoken.enabled = enabled;
  };

  // Configure event listener to enable/disable user-token when the current user changes
  Sonatype.Events.addListener('nexusStatus', function () {
    if (Sonatype.user.curr && Sonatype.user.curr.isLoggedIn) {
      // If logged in, then request status
      Ext.Ajax.request({
        url: Nexus.usertoken.resourcePath('/status'),
        method: 'GET',
        suppressStatus: true, // suppress all error status

        success: function (response) {
          var status = Ext.decode(response.responseText);
          Nexus.usertoken.setEnabled(status.enabled);
        },

        failure: function () {
          // force back to disabled on any failure
          Nexus.usertoken.setEnabled(false);
        }
      });
    }
    else {
      // if not logged in reset back to false
      Nexus.usertoken.setEnabled(false);
    }
  });

});