/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/

// FIXME: Rename this module/class

define('Sonatype/repoServer/StagingPanel', function() {

    Sonatype.repoServer.StagingPanel = function(cfg) {
      var
            i, sp = Sonatype.lib.Permissions, toolbar = [],
            updateGridView,
            _this = this,
            config = cfg || {},
            defaultConfig = {
              title : 'Staging Profiles'
            };
      Ext.apply(this, config, defaultConfig);

      this.targetStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'resourceURI',
            fields : [{
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'contentClass'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repoTargets,
            autoLoad : false,
            listeners : {
              load : {
                fn : function(store, recs, opts) {
                  for (i = recs.length - 1; i >= 0; i--)
                  {
                    var rec = recs[i];
                    if (rec.get('contentClass') != 'maven1' && rec.get('contentClass') != 'maven2')
                    {
                      store.remove(rec);
                    }
                  }
                },
                scope : this
              }
            }
          });

      this.groupStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'format'
                }, {
                  name : 'userManaged'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.groups,
            autoLoad : false,
            listeners : {
              load : {
                fn : function(store, recs, opts) {
                  for (var i = recs.length - 1; i >= 0; i--)
                  {
                    var rec = recs[i];
                    if (!rec.get('userManaged'))
                    {
                      store.remove(rec);
                    }
                  }
                },
                scope : this
              }
            }
          });

      this.repoStore = new Ext.data.JsonStore({
            root : 'data',
            id : 'id',
            fields : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  sortType : Ext.data.SortTypes.asUCString
                }, {
                  name : 'format'
                }, {
                  name : 'repoType'
                }, {
                  name : 'repoPolicy'
                }],
            sortInfo : {
              field : 'name',
              direction : 'ASC'
            },
            url : Sonatype.config.repos.urls.repositories,
            autoLoad : false
          });

      this.modeStore = new Ext.data.SimpleStore({
            fields : ['value', 'display'],
            data : [['BOTH', 'Deploy and UI Upload'], ['DEPLOY', 'Deploy'], ['UPLOAD', 'UI Upload']]
          });

      this.fullModeStore = new Ext.data.SimpleStore({
            fields : ['value', 'display'],
            data : [['BOTH', 'Deploy and UI Upload'], ['DEPLOY', 'Deploy'], ['UPLOAD', 'UI Upload'], ['GROUP', 'Group']]
          });

      this.joinGroups = function(value, parent) {
        var s = "";
        for (var i = 0; i < value.length; i++)
        {
          if (i > 0)
          {
            s += ', ';
          }
          if (value[i])
          {
            var rec = this.groupStore.getAt(this.groupStore.find('id', value[i]));
            if (rec)
            {
              s += rec.get('name');
            }
          }
        }
        return s;
      };

      this.getRepositoryTarget = function(val, parent) {
        if (val)
        {
          var rec = this.targetStore.getAt(this.targetStore.find('id', val));
          if (rec)
          {
            return rec.get('name');
          }
        }
        return '';
      };

      this.getRepoName = function(value, parent) {
        if (value)
        {
          var rec = this.repoStore.getAt(this.repoStore.find('id', new RegExp('^' + value + '$')));
          if (rec)
          {
            return rec.get('name');
          }
        }
        return '';
      }

      this.getModeName = function(value, parent) {
        if (value)
        {
          var rec = this.fullModeStore.getAt(this.fullModeStore.find('value', value));
          if (rec)
          {
            return rec.get('display');
          }
        }
        return '';
      }

      if (sp.checkPermission('nexus:stagingprofileorder', sp.CREATE))
      {
        toolbar.push({
              id : 'profile-move-up-button',
              text : 'Move Up',
              icon : Sonatype.config.resourcePath + '/images/icons/move-up.gif',
              cls : 'x-btn-text-icon',
              tooltip : {
                title : 'Profile Ordering',
                text : 'Move the profile up in the list priority.'
              },
              handler : this.moveUpResourceHandler,
              scope : this
            }, {
              id : 'profile-move-down-button',
              text : 'Move Down',
              icon : Sonatype.config.resourcePath + '/images/icons/move-down.gif',
              cls : 'x-btn-text-icon',
              tooltip : {
                title : 'Profile Ordering',
                text : 'Move the profile down in the list priority.'
              },
              handler : this.moveDownResourceHandler,
              scope : this
            }, {
              id : 'profile-save-order-button',
              text : 'Save Order',
              icon : Sonatype.config.extPath + '/resources/images/default/dd/drop-yes.gif',
              cls : 'x-btn-text-icon',
              tooltip : {
                title : 'Profile Ordering',
                text : 'Save the current profile priority order.'
              },
              handler : this.saveOrderResourceHandler,
              scope : this
            });
      }

      /* NXCM-4478 begin
       * Prevent loading of main dataStore before dependent stores are loaded
       *
       * FIXME this should rather be implemented in the Sonatype.panels.GridViewer itself (superclass)
       */
      var mask;
      var latch = 0;
      var dataStoreLoader = function() {
        // only load if all dependent stores are already loaded
        latch = latch + 1;
        if ( latch >= _this.dataStores.length ) {
          _this.dataStore.load();
        }
      };

      var dataStores = [this.targetStore, this.groupStore, this.repoStore];
      for (var j = 0; j < dataStores.length; j++) {
        var store = dataStores[j];
        store.on('load', dataStoreLoader, _this, {single:true});
      }
      /* NXCM-4478 end*/

      Sonatype.repoServer.StagingPanel.superclass.constructor.call(this, {
            dataStores : dataStores,
            addMenuInitEvent : 'profileAddMenuInit',
            deleteButton : sp.checkPermission('nexus:stagingprofiles', sp.DELETE),
            rowClickEvent : 'profileViewInit',
            url : Sonatype.config.servicePath + '/staging/profiles',
            dataAutoLoad : false,
            tabbedChildren : true,
            tbar : toolbar,
            dataSortInfo : {
              field : 'order',
              direction : 'asc'
            },
            columns : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  header : 'Name',
                  width : 100,
                  renderer : function(value, metadata, record, rowIndex, colIndex, store) {
                    return record.get('mode') == 'GROUP' ? ('<b>' + value + '</b>') : value;
                  }
                }, {
                  name : 'mode'
                }, {
                  name : 'sMode',
                  header : 'Mode',
                  width : 150,
                  mapping : 'mode',
                  convert : this.getModeName.createDelegate(this)
                }, {
                  name : 'repositoryTargetId'
                }, {
                  name : 'sTarget',
                  header : 'Repo Target',
                  width : 100,
                  mapping : 'repositoryTargetId',
                  convert : this.getRepositoryTarget.createDelegate(this)
                }, {
                  name : 'currentRepositoryIds'
                },
                {
                  name : 'order',
                  sortType : Ext.data.SortTypes.asInt
                }, {
                  name : 'finishNotifyRoles'
                }, {
                  name : 'promotionNotifyRoles'
                }, {
                  name : 'promotionTargetRepository'
                }, {
                  name : 'deployURI'
                }, {
                  name : 'sPromotionTargetRepository',
                  header : 'Release Repository',
                  width : 200,
                  mapping : 'promotionTargetRepository',
                  convert : this.getRepoName.createDelegate(this)
                }, {
                  name : 'targetGroups'
                }, {
                  name : 'sTargetGroups',
                  header : 'Target Groups',
                  width : 150,
                  mapping : 'targetGroups',
                  convert : this.joinGroups.createDelegate(this),
                  autoExpand : true
                }
              , {
                  name: 'properties'
                }
            ]
          });

      /* NXCM-4478 begin
       * Prevent loading of main dataStore before dependent stores are loaded
       */
      // delay this listener to have the panel completely rendered before applying the loadmask
      // (ExtJS bug, event 'afterrender' not working)
      this.gridPanel.on('render', function() {
        if (!this.dataStore.isLoaded) {
          mask = new Ext.LoadMask(_this.gridPanel.el);
          mask.show();
        }
      }, _this, {single:true, delay : 100});


    //  _this.gridPanel.loadMask.show();
      this.dataStore.on('load', function() {
        if(mask) {
          mask.hide();
        }
      });
      /* NXCM-4478 end*/
    };

    Ext.extend(Sonatype.repoServer.StagingPanel, Sonatype.panels.GridViewer, {
          emptyText: 'No profiles defined',
          emptyTextWhileFiltering: 'No profiles matched criteria: {criteria}',

          moveUpResourceHandler : function() {
            if (this.gridPanel.getSelectionModel().hasSelection())
            {
              var rec = this.gridPanel.getSelectionModel().getSelected();

              if (rec)
              {
                var i = this.gridPanel.getStore().indexOf(rec);

                if (i > 0)
                {
                  this.gridPanel.getStore().remove(rec);
                  this.gridPanel.getStore().insert(i - 1, rec);
                  this.gridPanel.getSelectionModel().selectRecords([rec], false);
                }
              }
            }
          },
          moveDownResourceHandler : function() {
            if (this.gridPanel.getSelectionModel().hasSelection())
            {
              var rec = this.gridPanel.getSelectionModel().getSelected();

              if (rec)
              {
                var i = this.gridPanel.getStore().indexOf(rec);

                if (i < (this.gridPanel.getStore().getCount() - 1))
                {
                  this.gridPanel.getStore().remove(rec);
                  this.gridPanel.getStore().insert(i + 1, rec);
                  this.gridPanel.getSelectionModel().selectRecords([rec], false);
                }
              }
            }
          },
          saveOrderResourceHandler : function() {
            var idList = [];
            for (var i = 0; i < this.gridPanel.getStore().getCount(); i++)
            {
              var rec = this.gridPanel.getStore().getAt(i);

              if (rec.data.resourceURI != 'new')
              {
                idList[idList.length] = rec.data.id;
              }
            }

            Sonatype.utils.defaultToNo();
            Sonatype.MessageBox.show({
                  animEl : this.gridPanel.getEl(),
                  title : 'Save Profile Order',
                  msg : 'Save the current order of the Profiles?',
                  buttons : Sonatype.MessageBox.YESNO,
                  scope : this,
                  icon : Sonatype.MessageBox.QUESTION,
                  fn : function(btnName) {
                    if (btnName == 'yes' || btnName == 'ok')
                    {
                      Ext.Ajax.request({
                            callback : this.saveOrderCallback,
                            scope : this,
                            method : 'POST',
                            url : Sonatype.config.servicePath + '/staging/profile_order',
                            jsonData : {
                              data : idList
                            }
                          });
                    }
                  }
                });
          },
          saveOrderCallback : function(options, isSuccess, response) {
            if (!isSuccess)
            {
              Sonatype.utils.connectionError(response, 'The server did not save the order of the profiles.');
            }
          }
        });

    Sonatype.Events.addListener('nexusNavigationPostInit', function(nexusPanel) {

          nexusPanel.insert(2,{
                title : 'Build Promotion',
                id : 'st-enterprise-staging'
              });

          nexusPanel.add({
                enabled : Sonatype.lib.Permissions.checkPermission('nexus:stagingprofiles', Sonatype.lib.Permissions.READ),
                sectionId : 'st-enterprise-staging',
                title : 'Staging Profiles',
                tabId : 'stagingProfiles',
                tabCode : Sonatype.repoServer.StagingPanel
              });
        });

    Sonatype.Events.addListener('profileAddMenuInit', function(menu) {
          var sp = Sonatype.lib.Permissions;

          if (sp.checkPermission('nexus:stagingprofiles', sp.CREATE))
          {
            var createProfileFunc = function(container, rec, item, e) {
              rec.beginEdit();
              if (item.groupMode)
              {
                rec.set('mode', 'GROUP');
              }
              rec.set('resourceURI', 'new');
              rec.set('targetGroups', []);
              rec.set('finishNotifyRoles', []);
              rec.set('promotionNotifyRoles', []);
              rec.set('properties', {});
              rec.commit();
              rec.endEdit();
            };

            menu.add(['-', {
                  text : 'Staging Profile',
                  autoCreateNewRecord : true,
                  handler : createProfileFunc
                }]);

            menu.add(['-', {
                  text : 'Build Promotion Profile',
                  autoCreateNewRecord : true,
                  handler : createProfileFunc,
                  groupMode : true
                }]);
          }
        });

});
